#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
#
# Dieses Programm ist in Zusammenarbeit Referendare und ehemaligen Referendaren der Informatik 
# Studienseminare Hamm und Ansberg entstanden. Es steht unter der Creative Commons 
# by-nc-sa-Lizenz in der Version 3.0 ( http://creativecommons.org/licenses/by-nc-sa/3.0/de/ ).
# Folglich darf es beliebig kopiert und bearbeitet werden, sofern das Folgeprodukt wiederum unter 
# gleichen Lizenzbedingungen vertrieben und auf die ursprnglichen Urheber verwiesen wird. 
#
# Die Namensnennung der ursprnglichen Urheber auf Arbeitsblttern und Programmbeispielen fr 
# die Schlerinnen und Schler ist nicht erforderlich, wohl aber bei der Weitergabe der 
# Materialien (beispielsweise zu Unterrichts- oder Forschungszwecken).
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html

# Gedacht u. a. als Rahmenprogramm zur Implementierung von Sequenzdiagrammen

class BasketballTurnier:
    def __init__(self, datum, beginn, ende, veranstalter):
        self.datum = datum
        self.beginn = beginn
        self.ende = ende
        self.veranstalter = veranstalter
        self.spiele = None
        self.tabelle = None
    def gibDatum(self):
        return self.datum
    def gibBeginn(self):
        return self.beginn
    def gibEnde(self):
        return self.ende
    def gibVeranstalter(self):
        return self.veranstalter
    def gibTabelle(self):
        return self.tabelle
    
    def spieleEintragen(self, *spiele):
        self.spiele = spiele
    def mannschaftenEintragen(self, *mannschaften):
        self.mannschaften = mannschaften
        self.tabelle = Tabelle(mannschaften)
    def spieleAusgeben(self):
        for s in self.spiele:
            print s.gibInfo()
    def gibSieger(self):
        if self.gibTabelle() == None:
            print "Es gibt noch keinen Sieger."
            return None
        else:
            return self.tabelle.gibTabellenFuehrer()
    def gibSpielMitNummer(self, nr):
        if self.spiele == None:
            print "Dem Turnier wurden noch keine Spiele zugewiesen!"
        else:
            return self.spiele[nr]
    
def tabSort(te1, te2):
    vglSiege = cmp(te1.siege, te2.siege)
    if vglSiege != 0:
        return vglSiege
    vglPunktDiff = cmp(te1.punktDiff, te2.punktDiff)
    if vglPunktDiff != 0:
        return vglPunktDiff
    return cmp(te1.punktErzielt, te2.punktErzielt)

class TabellenEintrag:
    def __init__(self, mannschaft):
        self.mannschaft = mannschaft
        self.spiele = 0
        self.siege = 0
        self.punktDiff = 0
        self.punktErzielt = 0
    def aktualisieren(self, siege, punktDiff, punktErzielt):
        self.spiele += 1
        self.siege += siege
        self.punktDiff += punktDiff
        self.punktErzielt += punktErzielt
    def __str__(self):
        return self.mannschaft.gibNamen().ljust(20) + " Spiele: " + str(self.spiele) + ", Siege: " + str(self.siege).rjust(2) + ", Diff: " + str(self.punktDiff).rjust(3) + ", Punkte: " + str(self.punktErzielt).rjust(3)
    
class Tabelle:
    def __init__(self, mannschaften):
        self.eintraege = []
        for m in mannschaften:
            self.eintrag = TabellenEintrag(m)
            self.eintraege.append(self.eintrag)
    def ausgabe(self):
        self.eintraege.sort(tabSort)
        self.eintraege.reverse()
        for e in self.eintraege:
            print str(e)
    def ergebnisEintragen(self, spiel):
        self.sieger = spiel.gibSieger()
        if self.sieger != None:
            self.verlierer = spiel.gibVerlierer()
            self.punktDiff = spiel.gibPunkteSieger() - spiel.gibPunkteVerlierer()
            for e in self.eintraege:
                if e.mannschaft == self.sieger:
                    e.aktualisieren(1, self.punktDiff, spiel.gibPunkteSieger())
                if e.mannschaft == self.verlierer:
                    e.aktualisieren(-1, -self.punktDiff, spiel.gibPunkteVerlierer())
        else:
            for e in self.eintraege:
                if e.mannschaft == spiel.mannschaftA:
                    e.aktualisieren(0, 0, spiel.punkteA)
                if e.mannschaft == spiel.mannschaftB:
                    e.aktualisieren(0, 0, spiel.punkteB)
    def gibTabellenFuehrer(self):
        self.eintraege.sort(tabSort)
        self.eintraege.reverse()
        return self.eintraege[0].mannschaft
    
class BasketballSpiel:
    def __init__(self, turnier, beginn, ende, ort, mannschaftA, mannschaftB):
        self.turnier = turnier
        self.beginn = beginn
        self.ende = ende
        self.ort = ort
        self.mannschaftA = mannschaftA
        self.mannschaftB = mannschaftB
        self.punkteA = -1
        self.punkteB = -1
    def gibBeginn(self):
        return self.beginn
    def gibEnde(self):
        return self.ende
    def gibOrt(self):
        return self.ort
    def gibMannschaftA(self):
        return self.mannschaftA
    def gibMannschaftB(self):
        return self.mannschaftB
    
    def ergebnisEintragen(self, punkteA, punkteB):
        self.punkteA = punkteA
        self.punkteB = punkteB
        self.turnier.gibTabelle().ergebnisEintragen(self)
    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteB > self.punkteA:
            return self.mannschaftB
        else:
            return None
    def gibVerlierer(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftB
        elif self.punkteB > self.punkteA:
            return self.mannschaftA
        else:
            return None
    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA
    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())
    def gibInfo(self):
        self.info = "Spiel zwischen '" + self.mannschaftA.name + "' und '" + self.mannschaftB.name + "' in " + self.ort.name + " (" + self.beginn + "-" + self.ende + ").\n"
        if self.punkteA < 0:
            self.info += "  Das Ergebnis steht noch nicht fest."
        else:
            if self.gibSieger() != None:
                self.info += "  Gewonnen hat '" + self.gibSieger().name + "' mit " + self.gibErgebnisSieger() + "."
            else:
                self.info += "  Das Spiel endete unentschieden " + str(self.punkteA) + ":" + str(self.punkteB) + "."
        return self.info
    
class Mannschaft:
    def __init__(self, name, kurs, trikotfarbe):
        self.name = name
        self.kurs = kurs
        self.trikotfarbe = trikotfarbe
    def gibNamen(self):
        return self.name
    def gibKurs(self):
        return self.kurs
    def gibTrikotfarbe(self):
        return self.trikotfarbe
    def setzeNamen(self, neuerName):
        self.name = neuerName
    def setzeTrikotfarbe(self, neueFarbe):
        self.trikotfarbe = neueFarbe
    
class Sporthalle:
    def __init__(self, name, standort):
        self.name = name
        self.standort = standort
    def gibNamen(self):
        return self.name
    def gibStandort(self):
        return self.standort


if __name__ == "__main__":
    dieKaengurus = Mannschaft("Die Kngurus", "11d2", "gruen")
    crazyFrogs = Mannschaft("Crazy Frogs", "11d1", "blau")
    dieUeberflieger = Mannschaft("Die berflieger", "11d3", "gelb")
    dieNowitzkis = Mannschaft("Die Nowitzkis", "11d4", "grau")
    
    basketballTurnier = BasketballTurnier("16.08.2037", "9:30", "12:30", "Stdtisches Gymnasium Selm")
    
    halleA = Sporthalle("Halle A", "Altbau")
    halleB = Sporthalle("Halle B", "Neubau")
    
    spielAB = BasketballSpiel(basketballTurnier, "09:45", "10:15", halleA, dieKaengurus, dieNowitzkis)
    spielCD = BasketballSpiel(basketballTurnier, "10:00", "10:30", halleB, dieUeberflieger, crazyFrogs)
    spielAC = BasketballSpiel(basketballTurnier, "10:45", "11:15", halleA, dieKaengurus, dieUeberflieger)
    spielBD = BasketballSpiel(basketballTurnier, "11:00", "11:30", halleB, dieNowitzkis, crazyFrogs)
    spielAD = BasketballSpiel(basketballTurnier, "11:45", "12:15", halleA, dieKaengurus, crazyFrogs)
    spielBC = BasketballSpiel(basketballTurnier, "12:00", "12:30", halleB, dieNowitzkis, dieUeberflieger)
    
    basketballTurnier.spieleEintragen(spielAB, spielCD, spielAC, spielBD, spielAD, spielBC)
    basketballTurnier.mannschaftenEintragen(dieKaengurus, crazyFrogs, dieUeberflieger, dieNowitzkis)
    
    basketballTurnier.spieleAusgeben()
    print "-----"
    
    spielAB.ergebnisEintragen(32, 20)
    spielCD.ergebnisEintragen(24, 23)
    spielAC.ergebnisEintragen(31, 22)
    spielBD.ergebnisEintragen(15, 33)
    spielAD.ergebnisEintragen(26, 31)
    spielBC.ergebnisEintragen(27, 27)
    
    basketballTurnier.spieleAusgeben()
    print "-----"
    
    print "Gesamtsieger: ", basketballTurnier.gibSieger().gibNamen()
    basketballTurnier.gibTabelle().ausgabe()
