#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Autor: Jens Jacobi
# Entwickelt im Rahmen der gemeinsamen 
# Unterrichtsvorbereitung für den bedarfsdeckenden
# Unterricht in der Oberstufe.
#
# Fachseminare Informatik an den Studienseminaren für 
# das Lehramt an Gesamtschulen und Gymnasien der 
# Seminare Hamm und Arnsberg
# 
# Fachleiter: Prof. Dr. L. Humbert 
#
# Dieses Programm ist Teil eines Unterrichtsbeispiels und
# steht unter der Creative Commons by-nc-sa-Lizenz.
# Folglich darf es beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter dieser Lizenz 
# vertrieben wird.
# Eine kommerzielle Nutzung ist nicht erlaubt.
#
# Die detaillierten Lizenzbedingungen finden sich auf
# http://creativecommons.org/licenses/by-nc-sa/3.0/deed.de
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html

# Implementierung der Lösung des "Basketball-Problems" 
# in der Programmiersprache Python.
# Die mit "# ..." gekennzeichneten Stellen sind zu ergÃ¤nzen.

from datenstrukturen import Liste
from tabellensortierer import TabellenSortierer, tabSort


class BasketballTurnier:
    def __init__(self, datum, beginn, ende, veranstalter):
        self.setzeDatum(datum)
        self.setzeBeginn(beginn)
        self.setzeEnde(ende)
        self.setzeVeranstalter(veranstalter)
        
        self.spiele = None
        self.tabelleMannschaften = None
        
        
    def setzeDatum(self, dat):
        self.datum= dat
    def gibDatum(self):
        return self.datum

    def setzeBeginn(self, anf):
        self.beginn = anf
    def gibBeginn(self):
        return self.beginn

    def setzeEnde(self, end):
        self.ende = end
    def gibEnde(self):
        return self.ende

    def setzeVeranstalter(self, veranst):
        self.veranstalter = veranst
    def gibVeranstalter(self):
        return self.veranstalter

    def spieleEintragen(self, *spiele):
        self.spiele = spiele

    def mannschaftenEintragen(self, *mannschaften):
        self.tabelleMannschaften = Tabelle(mannschaften)

    def gibTabelle(self):
        return self.tabelleMannschaften
         
    def spieleAusgeben(self):
        for spiel in self.spiele:
            print spiel.gibInfo()

    def gibSieger(self):
        if self.gibTabelle() == None:
            print "Es gibt noch keinen Sieger."
            return None
        else:
            return self.gibTabelle().gibTabellenFuehrer()
    
    def gibSpielMitNummer(self, nr):
        if self.spiele == None:
            print "Dem Turnier wurden noch keine Spiele zugewiesen!"
        else:
            return self.spiele[nr]





class TabellenEintrag:
    def __init__(self, mannschaft):
        self.setzeMannschaft(mannschaft)
        self.setzeSpiele(0)
        self.setzeSiege(0)
        self.setzePunktDiff(0)
        self.setzePunkteErzielt(0)
    
    def setzeMannschaft(self, mschaft):
        self.mannschaft= mschaft

    def setzeSpiele(self, zahl):
        self.spiele= zahl
    def gibSpiele(self):
        return self.spiele        
    
    def setzeSiege(self, zahl):
        self.siege= zahl 
    def gibSiege(self):
        return self.siege

    def setzePunktDiff(self, zahl):  
        self.punktDiff= zahl
    def gibPunktDiff(self):
        return self.punktDiff
    
    def setzePunkteErzielt(self, zahl):    
        self.punkteErzielt= zahl
    def gibPunkteErzielt(self):
        return self.punkteErzielt
        
    def aktualisieren(self, siege, punktDiff, punkteErzielt):
        self.setzeSpiele(self.gibSpiele() + 1)
        self.setzeSiege(self.gibSiege() + siege)
        self.setzePunktDiff(self.gibPunktDiff() + punktDiff)
        self.setzePunkteErzielt(self.gibPunkteErzielt() + punkteErzielt)
    
    def __str__(self):
        return self.mannschaft.gibName().ljust(12) + "\t Spiele: " + str(self.gibSpiele()) + ", Siege: " + str(self.gibSiege()).rjust(2) + ", Diff: " + str(self.gibPunktDiff()).rjust(3) + ", Punkte: " + str(self.gibPunkteErzielt()).rjust(3)


class Tabelle:
    def __init__(self, mannschaften):
        self.eintraege = []
        for mannschaft in mannschaften:
            self.eintrag = TabellenEintrag(mannschaft)
            self.eintraege.append(self.eintrag)

    def ausgabe(self):
        self.eintraege.sort(tabSort)
        self.eintraege.reverse()
        for eintrag in self.eintraege:
            print str(eintrag)

    def ergebnisEintragen(self, spiel):
        self.sieger = spiel.gibSieger()
        if self.sieger != None:
            self.verlierer = spiel.gibVerlierer()
            self.punktDiff = spiel.gibPunkteSieger() - spiel.gibPunkteVerlierer()
            for eintrag in self.eintraege:
                if eintrag.mannschaft == self.sieger:
                    eintrag.aktualisieren(1, self.punktDiff, spiel.gibPunkteSieger())
                if eintrag.mannschaft == self.verlierer:
                    eintrag.aktualisieren(-1, -self.punktDiff, spiel.gibPunkteVerlierer())
        else:
            for eintrag in self.eintraege:
                if eintrag.mannschaft == spiel.mannschaftA:
                    eintrag.aktualisieren(0, 0, spiel.punkteA)
                if eintrag.mannschaft == spiel.mannschaftB:
                    eintrag.aktualisieren(0, 0, spiel.punkteB)

    def gibTabellenFuehrer(self):
        self.eintraege.sort(tabSort)
        self.eintraege.reverse()
        return self.eintraege[0].mannschaft


class BasketballSpiel:
    def __init__(self, turnier, beginn, ende, ort, mannschaftA, mannschaftB):
        self.setzeTurnier(turnier)
        self.setzeBeginn(beginn)
        self.setzeEnde(ende)
        self.setzeOrt(ort)
        self.setzeMannschaftA(mannschaftA)
        self.setzeMannschaftB(mannschaftB)
        self.setzePunkteA(-1)
        self.setzePunkteB(-1)
    
    def setzeTurnier(self, tur):
        self.turnier = tur

    def setzeBeginn(self, anf):
        self.beginn = anf
    def gibBeginn(self):
        return self.beginn

    def setzeEnde(self, end):
        self.ende = end
    def gibEnde(self):
        return self.ende


    def setzeOrt(self, wo):
        self.ort = wo
    def gibOrt(self):
        return self.ort

    def setzeMannschaftA(self, mA):
        self.mannschaftA = mA        
    def gibMannschaftA(self):
        return self.mannschaftA

    def setzeMannschaftB(self, mB):
        self.mannschaftB = mB
    def gibMannschaftB(self):
        return self.mannschaftB

    def setzePunkteA(self, pA):
        self.punkteA = pA
    def gibPunkteA(self):
        return self.punkteA
        
    def setzePunkteB(self, pB):
        self.punkteB = pB        
    def gibPunkteB(self):
        return self.punkteB

    def ergebnisEintragen(self, punkteA, punkteB):
        self.setzePunkteA(punkteA)
        self.setzePunkteB(punkteB)
        self.turnier.gibTabelle().ergebnisEintragen(self)
        
    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteB > self.punkteA:
            return self.mannschaftB
        else:
            return None
    
    def gibVerlierer(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftB
        elif self.punkteB > self.punkteA:
            return self.mannschaftA
        else:
            return None
    
    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA
    
    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())

    def gibSieger(self):
        if self.punkteA > self.punkteB:
            return self.mannschaftA
        elif self.punkteA < self.punkteB:
            return self.mannschaftB
        else:
            return None

    def gibVerlierer(self):
        if self.punkteA < self.punkteB:
            return self.mannschaftA
        elif self.punkteA > self.punkteB:
            return self.mannschaftB
        else:
            return None

    def gibPunkteSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteA
        else:
            return self.punkteB
    
    def gibPunkteVerlierer(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        if self.punkteA > self.punkteB:
            return self.punkteB
        else:
            return self.punkteA

    def gibErgebnisSieger(self):
        if self.punkteA < 0:
            return "Das Spiel hat noch nicht stattgefunden!"
        return str(self.gibPunkteSieger()) + ":" + str(self.gibPunkteVerlierer())

    def gibInfo(self):
        self.info = "Spiel zwischen '" + self.mannschaftA.name + "' und '" + self.mannschaftB.name + "' in " + self.ort.name + " (" + self.beginn + "-" + self.ende + ").\n"
        if self.punkteA < 0:
            self.info += "  Das Ergebnis steht noch nicht fest."
        else:
            if self.gibSieger() != None:
                self.info += "  Gewonnen haben '" + self.gibSieger().name + "' mit " + self.gibErgebnisSieger() + "."
            else:
                self.info += "  Das Spiel endete unentschieden " + str(self.punkteA) + ":" + str(self.punkteB) + "."
        return self.info


class Mannschaft:
    def __init__(self, name, kurs, trikotfarbe):
        self.setzeName(name)
        self.__setzeKurs(kurs)
        self.setzeTrikotfarbe(trikotfarbe)

    def setzeName(self, neuerName):
        self.name = neuerName
    def gibName(self):
        return self.name

    def __setzeKurs(self,ku):
        self.kurs= ku
    def gibKurs(self):
        return self.kurs

    def setzeTrikotfarbe(self, neueFarbe):
        self.trikotfarbe = neueFarbe
    def gibTrikotfarbe(self):
        return self.trikotfarbe


    
class Sporthalle:
    def __init__(self, name, standort):
        self.setzeName(name)
        self._setzeStandort(standort)

    def setzeName(self, neuerName):
        self.name = neuerName
    def gibName(self):
        return self.name

    def _setzeStandort(self, neuerStandort):
        self.standort = neuerStandort
    def gibStandort(self):
        return self.standort


if __name__ == "__main__":
    basketballTurnier = BasketballTurnier("16.08.2037", "9:30", "12:30", "Willy-Brandt-Gesamtschule Bergkamen")

    dieKaengurus = Mannschaft("Die Kängurus", "11d2", "gruen")
    crazyFrogs = Mannschaft("Crazy Frogs", "11d1", "blau")
    dieUeberflieger = Mannschaft("Die Überflieger", "11d3", "gelb")
    dieNowitzkis = Mannschaft("Die Nowitzkis", "11d4", "grau")

    halleA = Sporthalle("Halle A", "Altbau")
    halleB = Sporthalle("Halle B", "Neubau")

    spielAB = BasketballSpiel(basketballTurnier, "09:45", "10:15", halleA, dieKaengurus, dieNowitzkis)
    spielCD = BasketballSpiel(basketballTurnier, "10:00", "10:30", halleB, dieUeberflieger, crazyFrogs)
    spielAC = BasketballSpiel(basketballTurnier, "10:45", "11:15", halleA, dieKaengurus, dieUeberflieger)
    spielBD = BasketballSpiel(basketballTurnier, "11:00", "11:30", halleB, dieNowitzkis, crazyFrogs)
    spielAD = BasketballSpiel(basketballTurnier, "11:45", "12:15", halleA, dieKaengurus, crazyFrogs)
    spielBC = BasketballSpiel(basketballTurnier, "12:00", "12:30", halleB, dieNowitzkis, dieUeberflieger)
    
    basketballTurnier.spieleEintragen(spielAB, spielCD, spielAC, spielBD, spielAD, spielBC)
    basketballTurnier.mannschaftenEintragen(dieKaengurus, crazyFrogs, dieUeberflieger, dieNowitzkis)

    basketballTurnier.spieleAusgeben()
    print "-----"
    
    spielAB.ergebnisEintragen(32, 20)
    spielCD.ergebnisEintragen(24, 23)
    spielAC.ergebnisEintragen(31, 22)
    spielBD.ergebnisEintragen(15, 33)
    spielAD.ergebnisEintragen(26, 31)
    spielBC.ergebnisEintragen(27, 27)
    
    basketballTurnier.spieleAusgeben()
    print "-----"
    
    print "Gesamtsieger: ", basketballTurnier.gibSieger().gibName()
    basketballTurnier.gibTabelle().ausgabe()


