#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
#
# Dieses Programm ist in Zusammenarbeit Referendare und ehemaligen Referendaren der Informatik 
# Studienseminare Hamm und Ansberg entstanden. Es steht unter der Creative Commons 
# by-nc-sa-Lizenz in der Version 3.0 ( http://creativecommons.org/licenses/by-nc-sa/3.0/de/ ).
# Folglich darf es beliebig kopiert und bearbeitet werden, sofern das Folgeprodukt wiederum unter 
# gleichen Lizenzbedingungen vertrieben und auf die ursprnglichen Urheber verwiesen wird. 
#
# Die Namensnennung der ursprnglichen Urheber auf Arbeitsblttern und Programmbeispielen fr 
# die Schlerinnen und Schler ist nicht erforderlich, wohl aber bei der Weitergabe der 
# Materialien (beispielsweise zu Unterrichts- oder Forschungszwecken).
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html

class Kino:
	"""Klasse Kino"""
	def __init__(self, name, ort):
		"""Konstruktor -- Argumente: \"name\", \"ort\" """
		self.name = name
		self.ort = ort
		self.saal4 = Saal("Saal 4", 334)
		self.saal6 = Saal("Saal 6", 128)
		self.kasse = Kasse()
	def gibNamen(self):
		return self.name
	def gibOrt(self):
		return self.ort
	def gibKasse(self):
		return self.kasse
	def gibSaal4(self):
		return self.saal4
	def gibSaal6(self):
		return self.saal6

class Kinobesucher:
	"""Klasse Kinobesucher"""
	def __init__(self, name, kino):
		"""Konstruktor -- Argumente: \"name\", \"kino\" """
		self.name = name
		self.ticket = None
		self.kino = kino
	def gibNamen(self):
		return self.name
	def gibTicket(self):
		return self.ticket
	def ticketAbholen(self, film):
		"""Argument: \"film\" """
		self.ticket = self.kino.gibKasse().ticketAusgeben(film)
	def ticketUmtauschen(self, neuerFilm):
		"""Argument: \"neuerFilm\" """
		self.ticket = self.kino.gibKasse().ticketUmtauschen(self.ticket, neuerFilm)

class Film:
	"""Klasse Film"""
	def __init__(self, name, startzeit):
		"""Konstruktor -- Argumente: \"name\", \"startzeit\" """
		self.name = name
		self.startzeit = startzeit
	def gibNamen(self):
		return self.name
	def gibStartzeit(self):
		return self.startzeit
	def gibSaal(self):
		return self.saal
	def setzeSaal(self, saal):
		self.saal = saal
		
class Saal:
	"""Klasse Saal"""
	def __init__(self, name, sitzplaetze):
		"""Konstruktor -- Argumente: \"name\", \"sitzplaetze\" """
		self.name = name
		self.sitzplaetze = sitzplaetze
		self.freiePlaetze = sitzplaetze
		self.film = None
	def gibFreiePlaetze(self):
		return self.freiePlaetze
	def erhoeheFreiePlaetzeUm(self, anzahl):
		self.freiePlaetze = self.freiePlaetze + anzahl
	def gibNamen(self):
		return self.name
	def setzeNamen(self, name):
		self.name = name
	def gibFilm(self):
		return self.film
	def setzeFilm(self, film):
		"""Argument: \"film\" (der Film, der in diesem Saal gezeigt wird)"""
		self.film = film

	def leeren(self):
		self.freiePlaetze = self.sitzplaetze

	def zeigeAusgabeFuerAnzeigetafel(self):
		print self.name + ": gespielt wird " + self.gibFilm().gibNamen() + " (" + str(self.freiePlaetze) + " von " + str(self.sitzplaetze) + " Plaetzen frei)."

class Kasse:
	"""Klasse Kasse"""
	def ticketAusgeben(self, film):
		"""Argument: \"film\" (der Film, fuer den ein Ticket gewuenscht wird)"""
		freiePlaetze = film.gibSaal().gibFreiePlaetze()
		if freiePlaetze > 0:
			film.gibSaal().erhoeheFreiePlaetzeUm(-1)
			return Ticket(film)

	def ticketUmtauschen(self, altesTicket, film):
		"""Argumente: \"altesTicket\", \"film\" (vernichtet \"altesTicket\" und gibt ein neues Ticket fuer \"film\" aus)"""
		altesTicket.gibFilm().gibSaal().erhoeheFreiePlaetzeUm(1)
		altesTicket = None
		return self.ticketAusgeben(film)

class Ticket:
	"""Klasse Ticket"""
	def __init__(self, film):
		"""Konstruktor -- Argument: \"film\" (erzeugt ein Ticket fuer den Film \"film\")"""
		self.film = film
		print "Es wurde ein Ticket ausgestellt fuer den Film " + self.film.gibNamen() + "."
	def __del__(self):
		print "Ein Ticket fuer den Film " + self.film.gibNamen() + " wurde zurueckgenommen."
	def gibFilm(self):
		return self.film
	def gibAufschrift(self):
		return "Ticket fuer den Film \"" + self.film.gibNamen() + "\" um " + self.film.gibStartzeit() + " in \"" + self.film.gibSaal().gibNamen()




if __name__ == "__main__":

	cinedome = Kino("CineDome", "Hinterschaffetten")
	fdk3 = Film("Fluch der Karibik 3", "19:30")
	mbmf = Film("Mr. Bean macht Ferien", "19.45")

	fdk3.setzeSaal(cinedome.gibSaal4())
	mbmf.setzeSaal(cinedome.gibSaal6())
	cinedome.gibSaal4().setzeFilm(fdk3)
	cinedome.gibSaal6().setzeFilm(mbmf)

	jannik = Kinobesucher("Jannik", cinedome)
	laura = Kinobesucher("Laura", cinedome)
	max = Kinobesucher("Max", cinedome)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()

	jannik.ticketAbholen(fdk3)
	laura.ticketAbholen(fdk3)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()
	
	max.ticketAbholen(mbmf)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()

	jannik.ticketUmtauschen(mbmf)
	laura.ticketUmtauschen(mbmf)

	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()